/*****************************************************************************
 *   rtc.c:  Realtime clock C file for Philips LPC214x Family Microprocessors
 *
 *   Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2005.10.01  ver 1.00    Prelimnary version, first Release
 *
*****************************************************************************/

/* modified by Martin Thomas

	- Option to use the 32kHz-XTAL as clock-source

	  from LPC2138 errata (hopefully taken into account correclty here)
Note.1: Increased power consumption from battery while RTC is running from the
	main 3.3V supply
Introduction: The RTC is powered by its own power supply pin,
	Vbat, which can be connected to a battery or to
	the same 3.3 volt supply used by the rest of the device.
Problem: If the VBAT is connected to an external battery, RTC
	will consume more power from the battery if the core is
	running and the selected clock sourceis the prescaler.
Work-around: Switch the clock source such that the RTC
	takes the clock from the 32 KHz oscillator that is
	connected to the RTCX1 and RTCX2 pins. After initialization
	of the RTC, clear the PCRTC bit in the PCONP register to
	switch off the peripheral clock (pclk) to RTC. Any further
	writes to the RTC would require this bit to be set. This
	will reduce the power consumed from VBAT.

	- global second_tick handled as counter increment-interrupt demo
	- handle increment- and alarm-interrupt in ISR
	- reset both interrupt-flags on "start"

20200711:Modified by Mhel
RTCInit(), to be consistent  with original lpc2138_uart0irq examples

*/
#include "types.h"
#include "LPC21xx.h"                       /* LPC21xx definitions */
#include "timer.h"
#include "armVIC.h"
#include "timerISR.h"

static inline void RTC_connect(void)
{
#ifdef RTC_USE_32KHZ
    //RTCCCR = (CCR_CLKEN|CCR_CLKSRC);
    if ( RTCCCR & CCR_CLKSRC)
    {
        PCONP |= (1UL<<9);
    }
#endif
}

static inline void RTC_disconnect(void)
{
    //PCONP=0x1815BE;
#ifdef RTC_USE_32KHZ
    //RTCCCR = (CCR_CLKEN|CCR_CLKSRC);
    if ( RTCCCR & CCR_CLKSRC)
    {
        PCONP &= ~(1UL<<9);
    }
#endif
}

/*****************************************************************************
** Function name:		RTCInit
**
** Descriptions:		Initialize RTC timer
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void cfgRTC( void )
{
    //alarm_on = 0;

    /*--- Initialize registers ---*/
    RTCAMR = 0;
    RTCCIIR = IMSEC; //interrupt on second
    RTCCCR = 0;
    //RTCILR = 3;
//set up prescaler
    RTCPREINT = PREINT_RTC;
    RTCPREFRAC = PREFRAC_RTC;
////initialize the interrupt vector
    VICIntSelect &= ~VIC_BIT(VIC_RTC);  // RTC selected as IRQ
    VICVectAddr10 = (uint32_t)rtcISR;    // address of the ISR
    VICVectCntl10 = VIC_ENABLE | VIC_RTC;
    VICIntEnable = VIC_BIT(VIC_RTC);    // RTC interrupt enabled



    return;
}

/*****************************************************************************
** Function name:		RTCStart
**
** Descriptions:		Start RTC timer
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void RTCStart( void )
{

    /* clear interrupt-flags alarm and increment */
    RTCILR |= (ILR_RTCCIF | ILR_RTCALF);

#ifdef RTC_USE_32KHZ
    RTCCCR = (CCR_CLKEN|CCR_CLKSRC);//0x11
    RTC_disconnect();
#else
//    /*--- Start RTC counters ---*/
    RTCCCR |= CCR_CLKEN;
#endif

    return;
}

/*****************************************************************************
** Function name:		RTCStop
**
** Descriptions:		Stop RTC timer
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void RTCStop( void )
{
    /*--- Stop RTC counters ---*/
    RTCCCR &= ~CCR_CLKEN;

    return;
}

/*****************************************************************************
** Function name:		RTC_CTCReset
**
** Descriptions:		Reset RTC clock tick counter
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void RTC_CTCReset( void )
{
    /*--- Reset CTC ---*/
    RTCCCR |= CCR_CTCRST;
    return;
}

/*****************************************************************************
** Function name:		RTCSetTime
**
** Descriptions:		Setup RTC timer value
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void RTCSetTime( RTCTime Time )
{
    RTC_connect();

    RTCSEC = Time.RTC_Sec;
    RTCMIN = Time.RTC_Min;
    RTCHOUR = Time.RTC_Hour;
    RTCDOM = Time.RTC_Mday;
    RTCDOW = Time.RTC_Wday;
    RTCDOY = Time.RTC_Yday;
    RTCMONTH = Time.RTC_Mon;
    RTCYEAR = Time.RTC_Year;

    RTC_disconnect();

    return;
}

/*****************************************************************************
** Function name:		RTCSetAlarm
**
** Descriptions:		Initialize RTC timer
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void RTCSetAlarm( RTCTime Alarm )
{
    RTC_connect();

    RTCALSEC = Alarm.RTC_Sec;
    RTCALMIN = Alarm.RTC_Min;
    RTCALHOUR = Alarm.RTC_Hour;
    RTCALDOM = Alarm.RTC_Mday;
    RTCALDOW = Alarm.RTC_Wday;
    RTCALDOY = Alarm.RTC_Yday;
    RTCALMON = Alarm.RTC_Mon;
    RTCALYEAR = Alarm.RTC_Year;

    RTC_disconnect();

    return;
}

/*****************************************************************************
** Function name:		RTCGetTime
**
** Descriptions:		Get RTC timer value
**
** parameters:			None
** Returned value:		The data structure of the RTC time table
**
*****************************************************************************/
RTCTime RTCGetTime( void )
{
    RTCTime LocalTime;

    LocalTime.RTC_Sec = RTCSEC;
    LocalTime.RTC_Min = RTCMIN;
    LocalTime.RTC_Hour = RTCHOUR;
    LocalTime.RTC_Mday = RTCDOM;
    LocalTime.RTC_Wday = RTCDOW;
    LocalTime.RTC_Yday = RTCDOY;
    LocalTime.RTC_Mon = RTCMONTH;
    LocalTime.RTC_Year = RTCYEAR;
    return LocalTime;
}

/*****************************************************************************
** Function name:		RTCSetAlarmMask
**
** Descriptions:		Set RTC timer alarm mask
**
** parameters:			Alarm mask setting
** Returned value:		None
**
*****************************************************************************/
void RTCSetAlarmMask( uint32_t AlarmMask )
{
    RTC_connect();

    /*--- Set alarm mask ---*/
    RTCAMR = AlarmMask;

    RTC_disconnect();

    return;
}

/*****************************************************************************
** Function name:		cfgTimer1
**
** Descriptions:		Initialize Timer1
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void cfgTimer1(void){
/* Timer1 Initialization */
    T1PR =  T1PRESCALER;//
    T1MR0 = 1; //MatchRegister0 ON
    T1MCR = 3; //Interrupt on Match, Reset TimerCounter
    T1TCR = 1;
    T1CCR = 0;                            // disable compare registers
    T1EMR = 0;                            // disable external match register
    VICIntSelect &= ~VIC_BIT(VIC_TIMER1);  // TIMER1 selected as IRQ
    VICVectAddr1 = (uint32_t)timer1ISR;    // address of the ISR
    VICVectCntl1 = VIC_ENABLE | VIC_TIMER1;
    VICIntEnable = VIC_BIT(VIC_TIMER1);    // UART0 interrupt enabled
}

/*****************************************************************************
**                            End Of File
******************************************************************************/

